/* Applicable preprocessor definitions:
 * FLASH_PROGRAMMER_SAVE_DATA
 *   - Define it to save data prior to writing flash and for
 *     making a copy of data after writing to flash.
 *     If you define this, the flash programming operation
 *     will take a very very very long time as the JTAG UART
 *     is terribly slow to write data to file and is slow
 *     to read data from file.
 *
 * FLASH_PROGRAMMER_DEBUG_ENABLED
 *   - define this to get more verbose information
 */


#include "DDStructs.h"
#include "LatticeMico32CFI.h"
#include "LookupServices.h"
#include "stdio.h"
#include "LatticeMico32.h"
#include "stdlib.h"

const char *const FLASH_INFO_FILENAME = "flashprog.txt";
const char *const FLASH_BINARY_FILENAME = "flashprog.bin";
#define BUFFER_SIZE 	(1024)


/* Buffer to speed-up flash-programming */
unsigned int Buffer[BUFFER_SIZE];


/* function that reads data from a file and writes to flash */
int WriteFlashDataFromFile(CFIFlashDevCtx_t *flash, const char *filename, unsigned int byteOffset, unsigned int bytes)
{
	FILE *fp;
	int error = 0;
	int dotCounter;
	unsigned int DwordsToWrite;
	unsigned int DwordsRead;
	unsigned int TotalDwordsToWrite;
	unsigned int sOffset;
	unsigned int eOffset;
	unsigned int sLength;

	/* get quad-byte alignment and quad-byte multiple */
	byteOffset &= (~3);
	bytes += 3;
	bytes &= (~3);
	
	/* open the file in binary mode */
	fp = fopen(filename, "rb");
	if(fp == 0){
		printf("failed to open file %s\n", filename);
		return(1);
	}

	sOffset = byteOffset;
	eOffset = sOffset + (bytes -1);

	/* fetch sector information for the sector containing the final byte-offset */
	if(LatticeMico32CFIFlashSectorInfo(flash, eOffset, &eOffset, &sLength)!= 0){
		printf("error: failed to get sector info for final address\n");
	}
	eOffset += sLength;

	/* beginning from the byte-offset specified, erase affected sectors */
	do{
		/* get sector information for the affected offset */
		if(LatticeMico32CFIFlashSectorInfo(flash, sOffset, &sOffset, &sLength)!= 0){
			printf("error: failed to get sector info for offset 0x%x\n", sOffset);
			error = 1;
			break;
		}

		/* erase affected sector */
		printf("erasing sector at offset 0x%x\n", sOffset);
		if(LatticeMico32CFIFlashEraseBlock(flash, sOffset)!= 0){
			printf("error: failed to erase sector at offset 0x%x\n", sOffset);
			error =1 ;
		}

		/*
		 * Move to starting-address of the next offset unless we've erased
		 * all affected sectors
		 */
		sOffset += sLength;
		if(sOffset >= eOffset)
			break;

	}while(1);
	printf("done erasing affected sectors\n");


	/* check for any errors */
	if(error != 0){
		fclose(fp);
		printf("errors erasing affected flash sectors - abort!! \n");
		return(1);
	}


	/* now read data from file and program the flash */
	sOffset = byteOffset;
	TotalDwordsToWrite = bytes >> 2;

	printf("starting to write total %d bytes\n", (TotalDwordsToWrite * 4));
	printf("Note: each dot represents %d bytes\n", BUFFER_SIZE*4 );
	dotCounter = 0;
	do{
		if(++dotCounter >16){
			printf("\n");
			dotCounter = 0;
		}
		printf(".");
		fflush(stdout);


		/* determine how many bytes to write based on our buffer */
		if(TotalDwordsToWrite > BUFFER_SIZE)
			DwordsToWrite = BUFFER_SIZE;
		else
			DwordsToWrite = TotalDwordsToWrite;

		/* read data from file */
#ifdef FLASH_PROGRAMMER_DEBUG_ENABLED
		printf("reading %d bytes from file\n", (DwordsToWrite*4));
#endif
		DwordsRead = fread(Buffer, sizeof(unsigned int), DwordsToWrite, fp);
		if(DwordsRead != DwordsToWrite){
			printf("error: failed to read sufficient data from programming file\n");
			error = 1;
			break;
		}

#ifdef FLASH_PROGRAMMER_DEBUG_ENABLED
		/* program this chunk of data */
		printf("writing %d bytes at offset 0x%x\n", (DwordsRead*4), sOffset);
#endif
		if(LatticeMico32CFIFlashWrite(flash, sOffset, (unsigned char *)Buffer, (DwordsRead*4))!= 0){
			printf("error: failed to program data\n");
			error = 1;
			break;
		}

		/* increment byte-offset for programming next chunk */
		sOffset += (DwordsRead * 4);

		/* decrement total dwords to write */
		TotalDwordsToWrite -= DwordsToWrite;
		if(TotalDwordsToWrite == 0)
			break;
	}while(1);
	printf("\n");
	/* close the file */
	fclose(fp);

	/* check if any errors happened when programming data */
	if(error != 0){
		printf(" errors during programming data - exit!!\n");
		return(1);
	}

	/* all done */
	return(0);
}


/* function that loads data from flash and saves it to a file */
int SaveFlashDatatoFile(CFIFlashDevCtx_t *flash, const char *filename, unsigned int byteOffset, unsigned int bytes)
{
	int i;
	int dotCounter;
	FILE *fp;
	unsigned int TotalDWordsToRead;
	unsigned int DwordsToRead;


	volatile unsigned int *pAddr;

	/* align offset to a quad-byte boundary */
	byteOffset &= (~0x3);
	TotalDWordsToRead = (bytes >> 2);

	/* open a file */
	fp = fopen(filename, "wb");
	if(fp == 0){
		printf("failed to open file %s\n", filename);
		return(1);
	}

	/* flush data-cache */
	MicoFlushDataCache();

	printf("preparing to upload %d bytes from offset 0x%x\n", (TotalDWordsToRead*4), byteOffset);
	printf("Note: each dot represents %d bytes\n", BUFFER_SIZE*4 );
	dotCounter = 0;

	pAddr = (volatile unsigned int *)(flash->base + byteOffset);
	while(1){
		if(++dotCounter >16){
			printf("\n");
			dotCounter = 0;
		}
		printf(".");
		fflush(stdout);

		/* calculate dwords to read based on available buffer */
		if(TotalDWordsToRead > BUFFER_SIZE)
			DwordsToRead = BUFFER_SIZE;
		else
			DwordsToRead = TotalDWordsToRead;

#ifdef FLASH_PROGRAMMER_DEBUG_ENABLED
		/* read flash-data into a temporary buffer */
		printf("reading %d bytes from flash\n", (DwordsToRead*4));
#endif
		for(i = 0; i < DwordsToRead; i++, pAddr++)
			Buffer[i] = *pAddr;

#ifdef FLASH_PROGRAMMER_DEBUG_ENABLED
		/* upload this data to the file */
		printf("uploading to file\n");
#endif
		fwrite(Buffer, sizeof(unsigned int), DwordsToRead, fp);

		TotalDWordsToRead -= DwordsToRead;
		if(TotalDWordsToRead == 0)
			break;
	}
	printf("\n");
	fclose(fp);

	/* all done */
	return(0);
}


/* function that reads flash-programming info file */
unsigned int GetFlashInfo(unsigned int *flashAddr, unsigned int *bytes)
{
	FILE *fp;
	char sLine[16];
	unsigned int uiAddr;
	unsigned int uiBytes;

	/* open programming-information file */
	fp = fopen(FLASH_INFO_FILENAME, "rt");
	if(fp == 0){
		printf("error opening info file %s\n", FLASH_INFO_FILENAME);
		return(1);
	}


	/*
	 * We expect the following information:
	 *  line 1: flash address where to burn (hexadecimal)
	 *  line 2: bytes to burn (must be a multiple of 4) (decimal)
	 */

	/* retrieve offset information */
	fgets(sLine, 15, fp);
	uiAddr = strtoul(sLine, 0, 16);


	/* retrieve bytes information */
	fgets(sLine, 15, fp);
	uiBytes = strtoul(sLine, 0, 10);


	/* return extracted information */
	*flashAddr = uiAddr;
	*bytes = uiBytes;

	return(0);
}

int main(void)
{
	unsigned int progAddr;
	unsigned int totalBytes;
	CFIFlashDevCtx_t *flash;
	DevFindCtx_t Find;

	/*
	 * retrieve programming information
	 */
	if(GetFlashInfo(&progAddr, &totalBytes) != 0){
		printf("failed to get programming information\n");
		return(1);
	}

	/*
	 * Find a flash device that has the desired address-range
	 */
	flash = (CFIFlashDevCtx_t *)MicoGetFirstDev("CFIFlashDevice", &Find);

	/*
	 * Go through all registered flash devices and see which one has the
	 * address range of interest
	 */
	do{
		if(flash == 0){
			printf("failed to find a suitable registered flash device\n");
			return(1);
		}
		if((flash->base <= progAddr) && (flash->end > progAddr)){
			/* we've found the flash-device of interest */
			break;
		}
		flash = (CFIFlashDevCtx_t *)MicoGetNextDev(&Find);
	}while(1);


	/*
	 * Make sure programming address and bytes to program seem okay
	 * w.r.t flash layout
	 */
	if( (progAddr < flash->base) ||
		((progAddr + totalBytes) >= flash->end) ){
		printf("flash programming information seems incorrect: \n");
		printf("Flash base: 0x%08x\n", flash->base);
		printf("Flash end : 0x%08x\n", flash->end);
		printf("Prog start: 0x%08x\n", progAddr);
		printf("Prog bytes: %d\n", totalBytes);
		return(1);
	}

	/*
	 * Make sure total bytes is a multiple of a word (4 bytes)
	 */
	if((totalBytes & 0x3)!= 0){
		printf("total bytes %d is not a multiple of 4 bytes\n", totalBytes);
		return(1);
	}


	/* convert absolute flash address to flash-relative offset */
	progAddr = progAddr - flash->base;

#ifdef FLASH_PROGRAMMER_SAVE_DATA

	/* read data from flash before erasing it */
	printf("saving data prior to programming, to readdata.bin\n");
	SaveFlashDatatoFile(flash, "readdata.bin", progAddr, totalBytes);

#else
	/* 
	 * inform the user that flash-data is erased without backing it up
	 */
	printf("Info: Flash programmer does not back-up flash data as default\n");
	printf("Info: To enable backup prior to erasing/writing to flash, \n");
	printf("      please define FLASH_PROGRAMMER_SAVE_DATA preprocessor\n");
	printf("      definition and recompile FlashProgrammer \n");

#endif


	/* write binary data from file to flash and provide offset */
	printf("programming flash with new data, %d total bytes at offset 0x%x\n", totalBytes, progAddr);
	if(WriteFlashDataFromFile(flash, FLASH_BINARY_FILENAME, progAddr, totalBytes) == 0){
#ifdef FLASH_PROGRAMMER_SAVE_DATA
		/* readback flash data after programming it */
		printf("reading back data to readback.bin\n");
		SaveFlashDatatoFile(flash, "readback.bin", progAddr, totalBytes);
#endif
		printf("done!\n");
	}

	/* all done! */
	return(0);
}

